#!/usr/bin/env python3
# -*- coding:utf-8 -*-
#
# Polychromatic is free software: you can redistribute it and/or modify
# it under the temms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Polychromatic is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Polychromatic. If not, see <http://www.gnu.org/licenses/>.
#
# Copyright (C) 2015-2016 Terry Cain <terry@terrys-home.co.uk>
#               2015-2016 Luke Horwell <lukehorwell37+code@gmail.com>
#

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Gdk', '3.0')
gi.require_version('AppIndicator3', '0.1')
from gi.repository import Gtk, Gdk, AppIndicator3 as appindicator
import collections, os
import sys
import signal

import razer.daemon_dbus
import razer.keyboard
import polychromatic.preferences
import polychromatic.profiles


class AppIndicator:
    """
    Simple indicator applet which sits in the tray.
    """

    @staticmethod
    def colour_to_hex(colour):
        """
        Converts a tuple to #RRGGBB format

        :param colour: Tuple or list of 3 integers
        :type colour: tuple or list

        :return: Hex String
        :rtype: str
        """
        return "#{0:02X}{1:02X}{2:02X}".format(*colour)

    def __init__(self):
        # Initialize the DBUS Daemon
        self.daemon = razer.daemon_dbus.DaemonInterface()

        # Store Colour and active effect
        self.colour = (255, 255, 255)
        self.secondary_colour = (0, 0, 0)
        self.active_effect = None

        # Read preferences and profiles.
        self.preferences = polychromatic.preferences.ChromaPreferences()
        self.profiles = polychromatic.profiles.ChromaProfiles(self.daemon)

        # Determine which icon to display.
        icon_type = self.preferences.get_pref('tray_applet','icon_type','system')
        icon_fallback = 'ibus-keyboard'
        if icon_type == 'system' or icon_type == '' or icon_type == None:
            icon = icon_fallback

        if icon_type == 'logo':
            # TODO: Retrieve icon relative to the program.
            logo_path = '/usr/share/razer_tray_applet/tray_icon.png'

        if icon_type == 'animated':
            # TODO: Retrieve icon relative to the program.
            logo_path = '/usr/share/razer_tray_applet/tray_icon_animated.gif'

        if icon_type == 'logo' or icon_type == 'animated':
            if os.path.exists(logo_path):
                icon = logo_path
            else:
                print('Default tray icon is missing at "' + logo_path + '". Using fallback.')
                icon = icon_fallback

        if icon_type == 'custom':
            icon_path = self.preferences.get_pref('tray_applet','icon_path')
            if not os.path.exists(icon_path):
                print('Custom icon "' + icon_path + '" could not be found. Using fallback.')
                icon = icon_fallback
            else:
                icon = icon_path

        if not icon:
            print('Malformed icon type: "' + icon_type + '"')
            icon = icon_fallback

        # Create the indicator
        self.ind = appindicator.Indicator.new("razer-keyboard-indicator", icon, appindicator.IndicatorCategory.APPLICATION_STATUS)
        self.ind.set_status (appindicator.IndicatorStatus.ACTIVE)

        # Create a menu
        self.menu = Gtk.Menu()

        # Create effects sub-menu
        effect_item = Gtk.MenuItem("Effects")
        effect_item.show()
        self.effect_menu = Gtk.Menu()
        self.effect_menu.show()
        effect_item.set_submenu(self.effect_menu)
        self.menu.append(effect_item)

        self.effect_menu_items = collections.OrderedDict()
        self.effect_menu_items["spectrum"] = Gtk.RadioMenuItem(group=None, label="Spectrum Effect")
        self.effect_menu_items["wave"] = Gtk.RadioMenuItem(group=self.effect_menu_items["spectrum"], label="Wave Effect")
        self.effect_menu_items["reactive"] = Gtk.RadioMenuItem(group=self.effect_menu_items["spectrum"], label="Reactive Effect")
        self.effect_menu_items["breath_r"] = Gtk.RadioMenuItem(group=self.effect_menu_items["spectrum"], label="Breath Effect (Random)")
        self.effect_menu_items["breath_s"] = Gtk.RadioMenuItem(group=self.effect_menu_items["spectrum"], label="Breath Effect (Single Colour)")
        self.effect_menu_items["breath_d"] = Gtk.RadioMenuItem(group=self.effect_menu_items["spectrum"], label="Breath Effect (Dual Colour)")
        self.effect_menu_items["static"] = Gtk.RadioMenuItem(group=self.effect_menu_items["spectrum"], label="Static Effect")
        self.effect_menu_items["none"] = Gtk.RadioMenuItem(group=self.effect_menu_items["spectrum"], label="No Effect (Off)")

        for button_key, button in self.effect_menu_items.items():
            button.connect("activate", self.menuitem_keyboard_effect_response, button_key)
            button.show()
            self.effect_menu.append(button)

        # Create brightness sub-menu
        brightness_item = Gtk.MenuItem("Brightness")
        brightness_item.show()
        self.brightness_menu = Gtk.Menu()
        self.brightness_menu.show()
        brightness_item.set_submenu(self.brightness_menu)
        self.menu.append(brightness_item)

        self.brightness_menu_items = collections.OrderedDict()
        self.brightness_menu_items[255] = Gtk.RadioMenuItem(group=None, label="Brightness 100%")
        self.brightness_menu_items[192] = Gtk.RadioMenuItem(group=self.brightness_menu_items[255], label="Brightness 75%")
        self.brightness_menu_items[128] = Gtk.RadioMenuItem(group=self.brightness_menu_items[255], label="Brightness 50%")
        self.brightness_menu_items[64] = Gtk.RadioMenuItem(group=self.brightness_menu_items[255], label="Brightness 25%")
        self.brightness_menu_items[0] = Gtk.RadioMenuItem(group=self.brightness_menu_items[255], label="Brightness 0%")

        for button_key, button in self.brightness_menu_items.items():
            button.connect("activate", self.menuitem_brightness_response, button_key)
            button.show()
            self.brightness_menu.append(button)

        # Create game mode sub-menu.
        game_mode_item = Gtk.MenuItem("Game Mode")
        game_mode_item.show()
        self.game_mode_menu = Gtk.Menu()
        self.game_mode_menu.show()
        game_mode_item.set_submenu(self.game_mode_menu)
        self.menu.append(game_mode_item)

        enable_game_mode_button = Gtk.MenuItem("Enable Game Mode")
        enable_game_mode_button.connect("activate", self.menuitem_enable_game_mode, True)
        enable_game_mode_button.show()
        self.game_mode_menu.append(enable_game_mode_button)

        disable_game_mode_button = Gtk.MenuItem("Disable Game Mode")
        disable_game_mode_button.connect("activate", self.menuitem_enable_game_mode, False)
        disable_game_mode_button.show()
        self.game_mode_menu.append(disable_game_mode_button)

        # Create profiles sub-menu.
        self.profiles_button = Gtk.MenuItem("Profiles")
        self.profiles_button.show()
        self.profiles_menu = Gtk.Menu()
        self.profiles_menu.show()
        self.profiles_button.set_submenu(self.profiles_menu)
        self.menu.append(self.profiles_button)
        self.refresh_profiles_menu()

        sep1 = Gtk.SeparatorMenuItem()
        sep1.show()
        self.menu.append(sep1)

        macro_button = Gtk.MenuItem("Enable Macro Keys")
        macro_button.connect("activate", self.menuitem_enable_macro_buttons_response, "macros")
        macro_button.show()
        self.menu.append(macro_button)

        sep2 = Gtk.SeparatorMenuItem()
        sep2.show()
        self.menu.append(sep2)

        self.primary_colour_button = Gtk.MenuItem("Change Primary Colour... ({0})".format(AppIndicator.colour_to_hex(self.colour)))
        self.primary_colour_button.connect("activate", self.set_static_colour, 1)
        self.secondary_colour_button = Gtk.MenuItem("Change Secondary Colour... ({0})".format(AppIndicator.colour_to_hex(self.secondary_colour)))
        self.secondary_colour_button.connect("activate", self.set_static_colour, 2)
        self.menu.append(self.primary_colour_button)
        self.menu.append(self.secondary_colour_button)
        self.primary_colour_button.show()
        self.secondary_colour_button.show()

        sep3 = Gtk.SeparatorMenuItem()
        sep3.show()
        self.menu.append(sep3)

        if os.path.exists('/usr/share/razer_chroma_controller/chroma_controller.py'):
            open_config_button = Gtk.MenuItem("Open Configuration Utility")
            open_config_button.connect("activate", self.menuitem_open_config)
            open_config_button.show()
            self.menu.append(open_config_button)

            sep4 = Gtk.SeparatorMenuItem()
            sep4.show()
            self.menu.append(sep4)
        else:
            print('Chroma Controller GUI not installed. Not integrating to menu.')

        quit_button = Gtk.MenuItem("Quit")
        quit_button.connect("activate", self.quit, "quit")
        quit_button.show()
        self.menu.append(quit_button)

        self.menu.show()
        self.ind.set_menu(self.menu)

        # If the user has preferences, set them.
        self.apply_startup_settings()

    # Shared functions for manipulating the device.
    def set_profile(self, profile_name):
        """ Shared function to send the profile to the keyboard """
        self.profiles.activate_profile_from_file(profile_name)
        self.profile_items['status'].set_label(profile_name)

    def set_effect(self, effect_type):
        """ Shared function to set an effect on the device """
        if effect_type == "breath_r":
            self.daemon.set_effect('breath', 1)
        elif effect_type == "breath_s":
            self.daemon.set_effect('breath', *self.colour)
        elif effect_type == "breath_d":
            self.daemon.set_effect('breath', self.colour[0], self.colour[1], self.colour[2], self.secondary_colour[0], self.secondary_colour[1], self.secondary_colour[2])
        elif effect_type == "none":
            self.daemon.set_effect('none', 1)
        elif effect_type == "reactive":
            self.daemon.set_effect('reactive', 1, *self.colour)
        elif effect_type == "spectrum":
            self.daemon.set_effect('spectrum')
        elif effect_type == "static":
            self.daemon.set_effect('static', *self.colour)
        elif effect_type == "wave":
            self.daemon.set_effect('wave', 1)

    # Tray Applet Responses
    def quit(self, widget, data=None):
        """
        Quits the application
        """
        Gtk.main_quit()

    def menuitem_keyboard_effect_response(self, widget, effect_type):
        """
        Click event for keyboard effects in the menu

        :param widget: MenuItem object
        :type widget: Gtk.RadioMenuItem

        :param effect_type: Type of effect
        :type effect_type: str
        """
        self.active_effect = effect_type
        if widget.get_active():
            self.set_effect(effect_type)

    def menuitem_brightness_response(self, widget, brightness):
        """
        Sets the brightness for the keyboard

        :param widget: MenuItem object
        :type widget: Gtk.RadioMenuItem

        :param brightness: Brightness value (scaled between 0-255)
        :type brightness: int
        """
        self.daemon.set_brightness(brightness)

    def menuitem_enable_macro_buttons_response(self, widget, string):
        """
        Enable macro keys

        :param widget: MenuItem object
        :type widget: Gtk.MenuItem

        :param string: Unused
        :type string: str
        """
        self.daemon.marco_keys(True)

    def menuitem_enable_game_mode(self, widget, enable):
        """
        Enables or disabled Game mode

        :param widget: MenuItem object
        :type widget: Gtk.MenuItem

        :param enable: Boolean
        :type enable: bool
        """
        self.daemon.game_mode(enable)

    def set_static_colour(self, widget, colour_index):
        """
        Sets the colour for effects.

        :param widget: MenuItem object
        :type widget: Gtk.MenuItem

        :param colour_index: index of colour
        :type colour_index: int
        """
        print("[Change Colour] Current: {0}".format(AppIndicator.colour_to_hex(self.colour)))

        # Create a colour selection dialog
        #colorsel = gtk.ColorSelection()
        color_selection_dlg = Gtk.ColorSelectionDialog('Change Static Colour')
        color_selection_result = color_selection_dlg.run()

        # If new colour is chosen.
        if color_selection_result == Gtk.ResponseType.OK:
            color_rgb = color_selection_dlg.get_color_selection().get_current_color()
            # Returns value between 0.0 - 1.0 * 255 = 8-bit RGB Value

            if colour_index == 1:
                self.colour = razer.keyboard.KeyboardColour.gdk_colour_to_rgb(color_rgb)
                self.primary_colour_button.set_label("Change Primary Colour... ({0})".format(AppIndicator.colour_to_hex(self.colour)))
                print("[Change Primary Colour] New: {0}".format(AppIndicator.colour_to_hex(self.colour)))
            else:
                self.secondary_colour = razer.keyboard.KeyboardColour.gdk_colour_to_rgb(color_rgb)
                self.secondary_colour_button.set_label("Change Secondary Colour... ({0})".format(AppIndicator.colour_to_hex(self.secondary_colour)))
                print("[Change Secondary Colour] New: {0}".format(AppIndicator.colour_to_hex(self.secondary_colour)))

            # If 'static', 'reactive' or 'breath' mode is set, refresh the effect.
            if self.active_effect == 'static':
                print("[Change Colour] Refreshing Static Mode")
                self.menuitem_keyboard_effect_response(self.effect_menu_items["static"], 'static')
            elif self.active_effect == 'reactive':
                print("[Change Colour] Refreshing Reactive Mode")
                self.menuitem_keyboard_effect_response(self.effect_menu_items["reactive"], 'reactive')
            elif self.active_effect == 'breath':
                print("[Change Colour] Refreshing Breath Mode")
                self.menuitem_keyboard_effect_response(self.effect_menu_items["breath"], 'breath')

        color_selection_dlg.destroy()

    def menuitem_open_config(self, widget):
        """
        Opens the Chroma Configuration Tool.
        """
        os.system('/usr/share/razer_chroma_controller/chroma_controller.py')

    def menuitem_set_profile(self, widget, profile_name):
        self.set_profile(profile_name)

        # FIXME: Something isn't right here, as it tries to load profile 'status'...
        # FIXME: Console text doesn't update until quit, previous error holding it up?
        #
        #~ Traceback (most recent call last):
          #~ File "../tray_applet/razer_tray_applet.py", line 343, in menuitem_set_profile
            #~ self.profiles.activate_profile_from_file(profile_name)
          #~ File "/usr/lib/python3/dist-packages/razer/profiles.py", line 136, in activate_profile_from_file
            #~ with open(os.path.join(self.preferences.SAVE_PROFILES, profile_name), 'rb') as profile_file:
        #~ FileNotFoundError: [Errno 2] No such file or directory: '/home/<user>/.config/razer_chroma/profiles/status'

    def refresh_profiles_menu(self):
        profile_list = self.profiles.get_profiles()

        # No profiles found.
        if not profile_list:
            self.profiles_status = Gtk.MenuItem("No profiles found.")
            self.profiles_status.show()
            self.profiles_menu.append(self.profiles_status)
            self.profiles_status.set_sensitive(False)
            return

        ## Display current profile in use.
        self.profile_items = collections.OrderedDict()
        self.profile_items['status'] = Gtk.RadioMenuItem(group=None, label="None Selected")
        self.profile_items['status'].show()
        self.profiles_menu.append(self.profile_items['status'])
        self.profile_items['status'].set_sensitive(False)

        sep = Gtk.SeparatorMenuItem()
        sep.show()
        self.profiles_menu.append(sep)

        # Populate the profiles list.
        for profile_name in self.profiles.get_profiles():
            self.profile_items[profile_name] = Gtk.RadioMenuItem(group=self.profile_items['status'], label=profile_name)

        for profile_name, button in self.profile_items.items():
            button.connect("activate", self.menuitem_set_profile, profile_name)
            button.show()
            self.profiles_menu.append(button)

    def apply_startup_settings(self):
        if self.preferences.get_pref('startup', 'enabled', 'false') == 'true':
            start_effect = self.preferences.get_pref('startup', 'start_effect', 'disabled')
            start_profile = self.preferences.get_pref('startup', 'start_profile', None)
            start_brightness = int(self.preferences.get_pref('startup', 'start_brightness', 0))
            start_macro = self.preferences.get_pref('startup', 'start_macro', 'false')
            start_col1_r = self.preferences.get_pref('primary_colors', 'red', 0)
            start_col1_g = self.preferences.get_pref('primary_colors', 'green', 255)
            start_col1_b = self.preferences.get_pref('primary_colors', 'blue', 0)
            start_col2_r = self.preferences.get_pref('secondary_colors', 'red', 255)
            start_col2_g = self.preferences.get_pref('secondary_colors', 'green', 0)
            start_col2_b = self.preferences.get_pref('secondary_colors', 'blue', 0)

            print('-- Applying start-up settings...')

            self.colour = (start_col1_r, start_col1_g, start_col1_b)
            self.secondary_colour = (start_col2_r, start_col2_g, start_col2_b)

            self.primary_colour_button.set_label("Change Primary Colour... ({0})".format(AppIndicator.colour_to_hex(self.colour)))
            print("[Change Primary Colour] New: {0}".format(AppIndicator.colour_to_hex(self.colour)))

            self.secondary_colour_button.set_label("Change Secondary Colour... ({0})".format(AppIndicator.colour_to_hex(self.secondary_colour)))
            print("[Change Secondary Colour] New: {0}".format(AppIndicator.colour_to_hex(self.secondary_colour)))

            if not start_effect == 'disabled':
                if start_effect == 'profile':
                    self.set_profile(start_profile)
                else:
                    self.set_effect(start_effect)

            if not start_brightness == 0:
                self.daemon.set_brightness(start_brightness)

            if start_macro == 'true':
                self.daemon.marco_keys(True)

            print('-- Start-up settings applied.')
            return
        else:
            return

def main():
    """
    Main function
    """
    # Kill the process when CTRL+C'd.
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    Gtk.main()
    sys.exit(0)

if __name__ == "__main__":
    indicator = AppIndicator()
    main()
